@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-import warnings
 import numpy as np
 import pandas as pd
 from covsirphy.cleaning.cbase import CleaningBase
+from covsirphy.util.error import deprecate
 
 
 class PopulationData(CleaningBase):
@@ -40,11 +40,18 @@ class PopulationData(CleaningBase):
         df = df.rename(
             {
                 "Country.Region": self.COUNTRY,
+                "Country/Region": self.COUNTRY,
                 "Province.State": self.PROVINCE,
+                "Province/State": self.PROVINCE,
                 "Population": self.N
             },
             axis=1
         )
+        # Confirm the expected columns are in raw data
+        expected_cols = [
+            self.COUNTRY, self.PROVINCE, self.N
+        ]
+        self.validate_dataframe(df, name="the raw data", columns=expected_cols)
         # ISO3
         if self.ISO3 not in df.columns:
             df[self.ISO3] = "-"
@@ -134,12 +141,15 @@ class PopulationData(CleaningBase):
             (int): population in the place
         """
         cleaned_df = self._cleaned_df.copy()
-        df = cleaned_df.loc[cleaned_df[self.ISO3] == country, :]
+        iso_df = cleaned_df.loc[cleaned_df[self.ISO3] == country, :]
+        c_df = cleaned_df.loc[cleaned_df[self.COUNTRY] == country, :]
+        df = pd.concat([iso_df, c_df], axis=0)
         if df.empty:
-            df = cleaned_df.loc[cleaned_df[self.COUNTRY] == country, :]
-            if df.empty:
+            if (cleaned_df[self.ISO3].unique()) == ["-"]:
                 raise KeyError(
-                    f"{country} is not registered. Please use ISO3 code, like JPN.")
+                    f"{country} is not registered. Please use registered country name.")
+            raise KeyError(
+                f"{country} is not registered. Please use ISO3 code as @country, like JPN.")
         if province is not None:
             df = df.loc[df[self.PROVINCE] == province, :]
             if df.empty:
@@ -167,14 +177,6 @@ class PopulationData(CleaningBase):
 
 
 class Population(PopulationData):
-    """
-    This is deprecated and please use PopulationData class.
-    """
-
+    @deprecate(old="Population()", new="PopulationData()")
     def __init__(self, filename):
         super().__init__(filename)
-        warnings.warn(
-            "Please use PopulationData() class rather than Population()",
-            DeprecationWarning,
-            stacklevel=2
-        )
